function [mu_w,gamma,gamma_used,iter,beta] = ...
    ARSBL(Phi,T, beta_ini, learnCorr, lambda, LearnLambda, MIN_GAMMA, MAX_ITERS, PRINT)
% EM based Sparse Bayesian Learning with multiple measurement vectors for temporally correlated sources.
% This is a slow version. For much more efficient version (more than 100
% times faster), see Reference [2] in the following.
%
% The key assumption is all the active sources have the same temporal correlation 
% (so beta is a scalar, not a vector with each element representing each source's 
% temporal correlation). However, experiments show  even in the case of different 
% sources having different AR(1) models, this algorithm outperforms the one
% assuming different sources have different AR(1) models. The possible reason is that
% in such high-dimensional problem the data could be better explained by source models 
% with the same AR model than actually present.
%
% ============= INPUT PARAMETERS =============
%   Phi         : N X M Dictionary matrix
%   T           : N X L Measurement matrix
%   beta_ini    : initial value for the temporal correlation. A scalar.
%   learnCorr   : if = 1: learn the beta (i.e. correlation);  if = 0: don't learn; just use beta_ini
%
%   lambda      : regularization parameter. Sometimes you can set it being the
%                 noise variance value, which leads to sub-optimal performance. 
%                 The optimal value is generally larger than the noise variance vlaue. 
%                 You need cross-validation methods or other methods to find it. 
%                 But for noiseless cases, setting lambda = 1e-10 leads to perfect performance.
%                 More information can be found in my blog post:
%                 http://marchonscience.blogspot.com/2011/02/misunderstandings-on-sparse-bayesian.html
%
%  LearnLambda  : If LearnLambda = 1, use the lambda as initial value and learn the optimal lambda 
%                 using the learning rule at Line 161-163. But note the
%                 learning rule is not robust if SNR <=20 dB. So in low SNR
%                 cases, to find a good value, you need use cross-validation or other methods such as
%                 those in Basis Pursuit Denoising. 
%                 If Learn_Lambda = 0, use the lambda as the final value.
%
%   MIN_GAMMA   : threshold for prunning small hyperparameters gamma_i.
%                 Generally, MIN_GAMMA = 1e-4 is a good value (especially
%                 in noisy cases).
%  MAX_ITERS    : maximum number of iterations
%  PRINT        : display flag. If = 1: show output; If = 0: supress output
%
% ============= OUTPUT PARAMETERS =============
%   mu_w          : Estimated solution matrix (or called source matrix),i.e. T = Phi * mu_w + noise. 
%                   Size of M X L. 
%   gamma_used    : indexes of nonzero elements in gamma
%   gamma         : Estimated gamma vector
%   iter          : Number of iterations used
%   beta          : learned beta value
%
% ============= See Also =============
%   SBL_demo
%
% ============= Reference =============
%   [1]Zhilin Zhang, B.D.Rao, Sparse Signal Recovery in the Presence of Correlated 
%      Multiple Measurement Vectors. ICASSP 2010
%   [2]Zhilin Zhang, Bhaskar D. Rao, Sparse Signal Recovery with Temporally Correlated
%      Source Vectors Using Sparse Bayesian Learning, submitted to IEEE
%      Journal of Selected Topics in Signal Processing, Special Issue on
%      Adaptive Sparse Representation of Data and Applications in Signal and
%      Image Processing, Oct.2010
%
% ============= Author =============
%   Zhilin Zhang (z4zhang@ucsd.edu)
%
% ============= Version =============
%   1.0 (06/28/2010)
%   1.1 (09/12/2010)  
%   1.2 (02/20/2011)  add more descriptions.



if PRINT
    if learnCorr,
        fprintf('\nRunning AR-SBL Algorithm via learning beta... \n');
    else
        fprintf('\nRunning AR-SBL Algorithm with given beta: %g \n', beta_ini);
    end
end




%% Transfer the multiple measurements into single measurement by vectorization
[N,L] = size(T);
M = size(Phi,2);
t = reshape(T',N*L,1);       
D = kron(Phi,eye(L));


%% Control parameters
MIN_Q = 1e-8;            % tolerance to stop the learning procedure.
betaGrid = [-0.99:0.02:-0.52, -0.5:0.1:0.5, 0.52:0.02:0.98];  % grid for search optimal beta


%% Initialization 
gamma = ones(M,1);
beta = beta_ini; 
for i = 1 : L, b(i) = beta^(i-1); end
B = toeplitz(b);
keep_list = [1:M]';
indexLength = length(keep_list);
mu = zeros(M*L,1);
iter = 0;
Q = -1;

%% EM iteration

while (1)
    iter = iter + 1; 
    if (PRINT), fprintf('\nIteration #%d\n',iter); end
    
    %% Prune weights as their hyperparameters go to zero
    if (min(gamma) < MIN_GAMMA)
        index = find(gamma > MIN_GAMMA);
        
        % Use the elements less than MAX_ALPHA to form new 'alpha'; the
        % pruned alpha_i are quitted from the computation.
        gamma = gamma(index);      
        Phi = Phi(:,index);
        
        % Find corresponding columns in D; remember each block corresponds to one alpha_i.
        indexLength = length(index);
        extIndex = zeros(indexLength,L);
        for i = 1 : indexLength
            extIndex(i,:) = (index(i)-1)*L+1 : index(i)*L;
        end
        extIndex = reshape(extIndex',indexLength*L,1);
        D = D(:,extIndex);        
        
        % post-processing
        keep_list = keep_list(index);
        if (indexLength == 0),  break;  end;
    end
    
    %% Compute new weights
    mu_old = mu;
    
    % =================== Method 1 ===================
    % For regularization parameter that is not too small. If there
    % exist some numerical issue, choose Method 2.
    Gamma = diag(gamma);
    Sigma0 = kron(Gamma,B);

    PG = Phi*Gamma;
    DS = kron(PG,B);
     
    Sigma_w = Sigma0 - DS' * inv(lambda*eye(N*L)+kron(PG*Phi',B)) * DS;
    mu = Sigma_w * D' * t /lambda;
    Cov_w = Sigma_w + mu * mu';

%     % ==================== Method 2 =================== 
%     invSigma0 = kron(diag(1./gamma),inv(B));
%     Sigma_w = inv(invSigma0 + D'*D/lambda);
%     
%     mu = Sigma_w * D' * t/lambda;
%     Cov_w = Sigma_w + mu * mu';
%     % =================================================

    % lambda learning rule. For low SNR cases, do NOT use the rule, since it
    % is not robust. See the comments in Line 20-30.
    if LearnLambda == 1
        lambda = (norm(t - D * mu,2)^2 + lambda * (indexLength*L - trace(Sigma_w * inv(Sigma0))))/(N*L);
    end

    % record gamma in previous iteration
    gamma_prevIter = gamma;

    if (~learnCorr)  % beta is known
        invB = inv(B);
        for i = 1 : indexLength
            Cxi = Cov_w( (i-1)*L+1:i*L , (i-1)*L+1:i*L );
            gamma(i) = trace(invB*Cxi)/L;
        end

    else  % beta needs to learn; 
        
            % learn the temporal correlation (beta)
            beta = learnBeta(beta,betaGrid,gamma,Cov_w,indexLength,L);

            % learn gamma 
            for i = 1 : L, b(i) = beta^(i-1); end
            B = toeplitz(b);  
            invB = inv(B);
            gamma = learnGamma(invB,indexLength,Cov_w,L);

    end

    
    %% check convergence condition
    if indexLength <= N
        Q_old = Q;
        invGamma = diag(1./gamma);
        dm = (indexLength*L)^2;
        T1 = reshape(kron(invGamma,invB)',1,dm);
        T2 = reshape(Cov_w,dm,1);
        Q = -log(prod(gamma)) * L/2 -indexLength*log(det(B)) - T1*T2;
        Qchange = abs(Q-Q_old);
        if Qchange <= MIN_Q
            if PRINT, fprintf(' Qchange <= MIN_Q, stop iterations\n\n'); end;
            break;
        end
    else
        Qchange = NaN;
    end
    
    if (PRINT)
        if (size(mu) == size(mu_old)), dmu = max(max(abs(mu_old - mu))); else dmu = NaN; end;
        gammaChange = max(abs(gamma-gamma_prevIter));
        fprintf(' Num Coeffs:%d;  gamma change:%g; mu change: %g; beta = %g; Q change: %g\n',...
            indexLength,gammaChange,dmu,beta,Qchange);
    end

    
    %% check maximum iteration condition
    if (iter >= MAX_ITERS), if PRINT, fprintf('Reach max iterations. Stop\n\n'); end; break;  end;

end;


%% Expand hyperparameters
gamma_used = sort(keep_list);

temp = zeros(M,1);
if (indexLength > 0),  
    temp(keep_list,1) = gamma;  
end;
gamma = temp;


%% Transfer to original weight matrix size (vec -> matrix)
mu_w = zeros(M,L); 
if (indexLength > 0) 
    for i = 1 : indexLength
        mu_w(keep_list(i),:) = mu((i-1)*L+1 : i*L)';
    end
end
 
if PRINT, fprintf('Running ends.\n'); end;
return;




%% =========================================================================
function gamma = learnGamma(invB,indexLength,C_x,L)

for i = 1 : indexLength
    Cxi = C_x( (i-1)*L+1:i*L , (i-1)*L+1:i*L );
    gamma(i,1) = trace(invB*Cxi)/L;
end




%% =========================================================================
function beta = learnBeta(beta_old,betaGrid,gamma,Cov_w,M,L)

invGamma = diag(1./gamma);
ind_old = find(betaGrid>=beta_old,1);
Q_old = evalQbeta(invGamma,beta_old,Cov_w,M,L);  
maxInd = length(betaGrid);

keepfind = 1;
if ind_old == maxInd
    Q_de = evalQbeta(invGamma,betaGrid(ind_old -1),Cov_w,M,L); 
    if Q_de <= Q_old
        keepfind = 0;
    else
        Q_old = Q_de;   ind_old = ind_old - 1;
    end
end

if ind_old == 1
    Q_in = evalQbeta(invGamma,betaGrid(ind_old + 1),Cov_w,M,L);  
    if Q_in > Q_old
        Q_old = Q_in;   ind_old = ind_old + 1;
    else
        keepfind = 0;
    end
end


while keepfind ==1 & ind_old < maxInd & ind_old > 1

    Q_in = evalQbeta(invGamma,betaGrid(ind_old + 1),Cov_w,M,L);  
    if Q_in > Q_old
        Q_old = Q_in;   ind_old = ind_old + 1;  % fprintf('   ind_increase: %d\n', ind_old);
    else
        Q_de = evalQbeta(invGamma,betaGrid(ind_old -1),Cov_w,M,L); 
        if Q_de <= Q_old
            keepfind = 0;  % find optimal value
        else
            Q_old = Q_de;   ind_old = ind_old - 1;  % fprintf('   ind_decrease: %d\n', ind_old);
        end
    end

end
beta = betaGrid(ind_old);



%%=============================================================
function Q = evalQbeta(invGamma,beta,Cov_w,M,L)
% evalute Q function of beta

for i = 1 : L, bta(i) = beta^(i-1); end; B = toeplitz(bta);
T1 = reshape(kron(invGamma,inv(B))',1,(M*L)^2);  
T2 = reshape(Cov_w,(M*L)^2,1);
Q = -M*log(det(B)) - T1*T2;  





