function [mu,gamma,gamma_used,k] = MSBL(Phi, T, lambda, Learn_Lambda, MIN_GAMMA, MAX_ITERS, PRINT)
% Sparse Bayesian Learning for Mulitple Measurement Vector (MMV) problems. 
% It can also be used for single measurement vector problem without any modification.
%
% ===== INPUTS =====
%   Phi         : N X M dictionary matrix
%   T           : N X L measurement matrix
%   lambda      : regularization parameter. Sometimes you can set it being the
%                 noise variance value, which leads to sub-optimal
%                 performance. The optimal value is generally larger than the
%                 noise variance vlaue. You need cross-validation methods or
%                 other methods to find it. But in noisless cases, setting
%                 lambda = 1e-10 leads to perfect performance.
%  Learn_Lambda : If Learn_Lambda = 1, use the lambda as initial value and learn the optimal lambda 
%                 using the learning rule at Line 106-109. But note the
%                 learning rule is not robust if SNR <=20 dB. So in low SNR
%                 cases, to find a good value, you need use cross-validation or other methods such as
%                 those in Basis Pursuit Denoising. 
%                 If Learn_Lambda = 0, use the lambda as the final value.
%
%   MIN_GAMMA   : threshold for prunning small hyperparameters gamma_i.
%                 Generally, MIN_GAMMA = 1e-4 is a good value (especially
%                 in noisy cases).
%  MAX_ITERS    : maximum number of iterations
%  PRINT        : display flag. If = 1: show output; If = 0: supress output
%
% ===== OUTPUTS =====
%   mu         : the estimated solution matrix, or called source matrix (size: M X L)
%   gamma      : final value of the M X 1 vector of hyperparameter values
%   gamma_used : indexes of nonzero gamma_i
%   k          : number of iterations used
%
%
% *** Reference ***
% [1] David P. Wipf, Bhaskar D. Rao, An Empirical Bayesian Strategy for Solving
%     the Simultaneous Sparse Approximation Problem, IEEE Trans. Signal
%     Processing, Vol.55, No.7, 2007.
% [2] Zhilin Zhang, Bhaskar D. Rao, Some Misunderstandings on the
%     Performance Evaluation of Sparse Bayesian Learning for Sparse Signal
%     Recovery. In Preparation. 2011.
%
%
% *** Author ***
%   Zhilin Zhang (z4zhang@ucsd.edu) 
%   (Modified based on David Wipf's original code)
%
% *** Version ***
%   1.1 (02/12/2011)
%
% *** See Also ***
%   SBL_demo
%
    


% *** Control parameters ***  
MIN_DMU             = 1e-8;     % threshold for stop iteration. In noiseless case, 1e-10 is good.

if (PRINT) fprintf('\nRunning MSBL ...\n'); end

% *** Initializations ***
[N M] = size(Phi); 
[N L] = size(T);  
gamma = ones(M,1); 
 
keep_list = [1:M]';
m = length(keep_list);
mu = zeros(M,L);
k = 0;               % iteration count


% *** Learning loop ***
while (1)

    % *** Prune weights as their hyperparameters go to zero ***
    if (min(gamma) < MIN_GAMMA )
        index = find(gamma > MIN_GAMMA);
        gamma = gamma(index);  % use all the elements larger than MIN_GAMMA to form new 'gamma'
        Phi = Phi(:,index);    % corresponding columns in Phi
        keep_list = keep_list(index);
        m = length(gamma);

        if (m == 0)   break;  end;
    end;


    mu_old =mu;
    Gamma = diag(gamma);
    G = diag(sqrt(gamma));
        
    % ****** estimate the solution matrix *****
    [U,S,V] = svd(Phi*G,'econ');
   
    [d1,d2] = size(S);
    if (d1 > 1)     diag_S = diag(S);
    else            diag_S = S(1);      end;
       
    Xi = G * V * diag((diag_S./(diag_S.^2 + lambda + 1e-16))) * U';
    mu = Xi * T;
    
    % ***** estimate the regularization parapeter lambda *****
    % You can use the following two lines of codes to estimate the lambda
    % when SNR >= 25 dB or in noiseless case. When SNR < 25dB, you'd better
    % use other method to estimate the lambda, since the following learning
    % rule is not robust in strongly noisy cases.
    if Learn_Lambda == 1
        PGP = Phi*Gamma*Phi';
        lambda = norm(T - Phi * mu,'fro')^2/(N*L) + lambda * trace(PGP*inv(lambda*eye(N)+PGP))/N;
    end;
    
    
    % *** Update hyperparameters, i.e. Eq(18) in the reference ***
    gamma_old = gamma;
    mu2_bar = sum(abs(mu).^2,2)/L;

    Sigma_w_diag = real( gamma - (sum(Xi'.*(Phi*Gamma)))');
    gamma = mu2_bar + Sigma_w_diag;


    % *** Check stopping conditions, etc. ***
    k = k+1;
    if (PRINT) disp(['iters: ',num2str(k),'   num coeffs: ',num2str(m), ...
            '   gamma change: ',num2str(max(abs(gamma - gamma_old)))]); end;
    if (k >= MAX_ITERS) break;  end;

    if (size(mu) == size(mu_old))
        dmu = max(max(abs(mu_old - mu)));
        if (dmu < MIN_DMU)  break;  end;
    end;

end;


% *** Expand weights, hyperparameters ***
gamma_used = sort(keep_list);

temp = zeros(M,1);
if (m > 0) temp(keep_list,1) = gamma;  end;
gamma = temp;

temp = zeros(M,L);
if (m > 0) temp(keep_list,:) = mu;  end;  
mu = temp;  

return;



