% Demo for the comparison of ARSBL and MSBL
%
% ARSBL are a slightly fast version of the one in:
%   [1]Zhilin Zhang, B.D.Rao, Sparse Signal Recovery in the Presence of Correlated 
%      Multiple Measurement Vectors. ICASSP 2010
% But for another much fast version (faster more than 100 times), see the
% paper:
%   [2]Zhilin Zhang, Bhaskar D. Rao, Sparse Signal Recovery with Correlated
%      Source Vectors Using Sparse Bayesian Learning, submitted to IEEE
%      Journal of Selected Topics in Signal Processing, Special Issue on
%      Adaptive Sparse Representation of Data and Applications in Signal and
%      Image Processing, Oct.2010
%
% Author: Zhilin Zhang (z4zhang@ucsd.edu)
% Feb.20, 2011


clear all; clc;

% problem dimension
N = 22;       % row number of the dictionary matrix 
M = N * 3;    % column number of the dictionary matrix
L = 3;        % number of measurement vectors
D = 10;       % diversity (number of active sources)
SNR = 25;     % SNR
iterNum = 20; % number of experiments


fprintf('\n\n==== Comparison between ARSBL and M-SBL. Results averaged over %d experiments ====\n',iterNum);

for it = 1 : iterNum
    fprintf('\n\nRunning %d:\n',it);

    % dictionary matrix with columns draw uniformly from the surface of a unit hypersphere
    Phi = randn(N,M);
    Phi = Phi./(ones(N,1)*sqrt(sum(Phi.^2)));
   
    % Generate D nonzero rows, each row being an AR(1) process. All the
    % AR(1) processes have different AR coefficients.
    beta = 1-rand(D,1)*0.2;            % true temporal correlations of each source
    nonzeroW(:,1) = randn(D,1);
    for i = 2 : L*100
        nonzeroW(:,i) = beta .* nonzeroW(:,i-1) + sqrt(1-beta.^2).*(ones(D,1).*randn(D,1));
    end
    nonzeroW = nonzeroW(:,end-L+1:end);   % ensure AR processes are stable

    % normalize along row
    nonzeroW = nonzeroW./( sqrt(sum(nonzeroW.^2,2)) * ones(1,L) );

    % The locations of nonzero rows are randomly chosen
    ind = randperm(M);
    indice = ind(1:D);
    Wgen = zeros(M,L);
    Wgen(indice,:) = nonzeroW;


    % noiseless signal
    signal = Phi * Wgen;

    % observation noise
    stdnoise = std(signal)*10^(-SNR/20);
    noise = randn(N,L).*(ones(N,1)*stdnoise);

    % noisy signal
    T = signal + noise;


    %============================ ARSBL ==========================

    beta_ini = 0.7;   % initial value for the correlation of each source
    learn_beta = 1;   % learn the correlation
    lambda = 1e-3;    % initial value for the regularization parameter. The two compared algorithms will 
                      % learn the optimal value in the demo. SEE details in
                      % the codes for the two algorithms.
    learnLambda = 1;  % using the learning rule to learn an optimal lambda. Note the learn rule is not robust 
                      % in low SNR cases (SNR <=25dB).
    MIN_GAMMA = 1e-4; % threshold for prunning small hyperparameters gamma_i 
    MAX_ITERS = 1000; % Maximum iteration times
    PRINT = 0; 
    
    tic;
    [Weight1,gamma1,gamma_used1,count1,beta_est1] ...
        = ARSBL(Phi,T, beta_ini, learn_beta, lambda, learnLambda, MIN_GAMMA, MAX_ITERS, PRINT);
    time1 = toc;
    TIME1(it) = time1;

    % failure rate: F1 = 1: perfect recovery; F1 = 0: worst recovery
    F1 = perfSupp(Weight1,indice,'firstlargest', D);               
    fail_ARSBL(it) = (F1~=1);                              
    
    % Mean Square Error (MSE)
    perf_ARSBL(it) = (norm(Wgen - Weight1,'fro')/norm(Wgen,'fro'))^2;  
    
    fprintf('  ARSBL: time = %5.2f; Findex = %3.2f, MSE = %3.2f%%; Fail_Rate = %4.3f%%; Ave-Time = %4.3f\n',...
        time1,F1,sum(perf_ARSBL)/it*100,sum(fail_ARSBL)/it*100,sum(TIME1)/it);
    %============================================================


    %============================ MSBL ==========================  
    tic;
    [Weight3,gamma_est3,gamma_used3,count3] = MSBL(Phi,T, lambda, learnLambda, MIN_GAMMA, MAX_ITERS, PRINT);
    time3 = toc;
    TIME3(it) = time3;

    % Failure rate
    F3 = perfSupp(Weight3,indice,'firstlargest', D);      
    fail_MSBL(it) = (F3~=1);      
    
    % MSE
    perf_MSBL(it) = (norm(Wgen - Weight3,'fro')/norm(Wgen,'fro'))^2;  
    
    fprintf('   MSBL: time = %5.2f; Findex = %3.2f, MSE = %3.2f%%; Fail_Rate = %4.3f%%; Ave-Time = %4.3f\n',...
        time3,F3,sum(perf_MSBL)/it*100,sum(fail_MSBL)/it*100,sum(TIME3)/it);
    %============================================================


end
fprintf('\n\n');
disp([' =========================== Experiment Settings =============================']);
disp(['         Dictionary Matrix Size : ',num2str(N),' x ',num2str(M)]);
disp(['  Number of Measurement Vectors : ',num2str(L)]);
disp(['      Number of Nonzero Sources : ',num2str(D)]);
disp(['                            SNR : ',num2str(SNR)]);
disp(['      Experiment Repeated Times : ',num2str(iterNum)]);
fprintf('\n');
disp([' =========================== Experiment Results ===============================']);
fprintf('        Failure Rate ( = 0: perfect ): ARSBL = %4.3f | MSBL = %4.3f\n',mean(fail_ARSBL),mean(fail_MSBL));
fprintf('                 MSE ( = 0: perfect ): ARSBL = %4.3f | MSBL = %4.3f\n',mean(perf_ARSBL),mean(perf_MSBL));





